@@ -1,16 +1,17 @@
 //! Module containing various utility functions.
 
 
+mod os;
 mod content_encoding;
 
 use std::f64;
 use std::cmp;
-use std::fs::File;
 use std::path::Path;
 use std::borrow::Cow;
 use rfsapi::RawFileData;
 use url::percent_encoding;
 use iron::headers::UserAgent;
+use std::fs::{FileType, File};
 use std::collections::HashMap;
 use time::{self, Duration, Tm};
 use iron::{mime, Headers, Url};
@@ -18,6 +19,7 @@ use std::io::{BufReader, BufRead};
 use base64::display::Base64Display;
 use mime_guess::{guess_mime_type_opt, get_mime_type_str};
 
+pub use self::os::*;
 pub use self::content_encoding::*;
 
 
@@ -115,7 +117,10 @@ pub fn uppercase_first(s: &str) -> String {
 /// assert!(!file_binary("Cargo.toml"));
 /// ```
 pub fn file_binary<P: AsRef<Path>>(path: P) -> bool {
-    File::open(path).and_then(|f| BufReader::new(f).read_line(&mut String::new())).is_err()
+    let path = path.as_ref();
+    path.metadata()
+        .map(|m| is_device(&m.file_type()) || File::open(path).and_then(|f| BufReader::new(f).read_line(&mut String::new())).is_err())
+        .unwrap_or(true)
 }
 
 /// Fill out an HTML template.
@@ -209,6 +214,11 @@ pub fn is_symlink<P: AsRef<Path>>(p: P) -> bool {
     p.as_ref().read_link().is_ok()
 }
 
+/// Check if a path refers to a file in a way that includes Unix devices and Windows symlinks.
+pub fn is_actually_file(tp: &FileType) -> bool {
+    tp.is_file() || is_device(tp)
+}
+
 /// Construct string representing a human-readable size.
 ///
 /// Stolen, adapted and inlined from [fielsize.js](http://filesizejs.com).
